#include <curses.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include "include/error.h"
#include "include/print.h"
#include "include/inotify_lib.h"

#define		INOTIFY_EVENTS\
	IN_MODIFY | IN_ATTRIB | IN_MOVED_FROM | IN_MOVED_TO |\
	IN_DELETE_SUBDIR | IN_DELETE_FILE | IN_CREATE_SUBDIR |\
	IN_CREATE_FILE

#define 	BUFSIZE		256

#define		NB_ARGS		2
#define		ARG_FILE	1

void usage()
{
	fprintf(stderr, "./inotify_app directory_to_watch\n");
}

int main(int argc, char **argv)
{
	fd_set rdfs;
	int fd_inotify;
	int c;
	char buffer[BUFSIZE];
	char *dir;

	if (argc != NB_ARGS) {
		usage();	
		return -EBADARG;
	}
	dir = argv[ARG_FILE];

	init_print();

	if (print_directory(dir) < 0) {
		cleanup_print();
		return -ERROR;
	}

	if ((fd_inotify = inotify_open()) < 0) {
		cleanup_print();
		perror("open()");
		return -EINOTIFY;
	}

	if (inotify_add_watch(fd_inotify, dir, INOTIFY_EVENTS)) {
		inotify_close(fd_inotify);
		cleanup_print();
		printf("Undetermined error.\n");
		return -EINOTIFY;
	}

	FD_ZERO(&rdfs);
	FD_SET(0, &rdfs);
	FD_SET(fd_inotify, &rdfs);

	while (select(fd_inotify+1, &rdfs, NULL, NULL, NULL)) {
		if (FD_ISSET(0, &rdfs)) {
			if ((c = getch()) == 'q') {
				break;
			}
		}
		else if (FD_ISSET(fd_inotify, &rdfs)) {
			if(print_directory(dir) < 0) {
				inotify_close(fd_inotify);
				cleanup_print();
				printf("Undetermined error.\n");
				return -ERROR;
			}
			read(fd_inotify, buffer, BUFSIZE) ;
		}

		FD_ZERO(&rdfs);
		FD_SET(0, &rdfs);
		FD_SET(fd_inotify, &rdfs);
	}

	inotify_close(fd_inotify);
	cleanup_print();

	return SUCCESS;
}
